package com.dbs.drnd.ark.cdc.http;

import java.io.Closeable;
import java.io.IOException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Base64;

import org.apache.http.HttpEntity;
import org.apache.http.HttpException;
import org.apache.http.HttpRequest;
import org.apache.http.HttpRequestInterceptor;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpUriRequest;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.protocol.HttpContext;
import org.apache.http.util.EntityUtils;

import com.dbs.drnd.ark.cdc.http.inter.ArkApi;
import com.dbs.drnd.ark.cdc.http.inter.Method;

/**
 * Ark CDC HTTP API
 * 
 * @author jinwoo
 *
 */
public class ArkHttpApi implements Closeable {

    private CloseableHttpClient httpClient;
    private String authorization = null;

    private ArkApi authInit;
    private ArkApi moduleStart;
    private ArkApi moduleStop;
    private ArkApi moduleState;

    private ArkApi peer;
    private ArkApi modules;
    private ArkApi diskFree;
    private ArkApi statistics;
    private ArkApi tracingFileList;
    private ArkApi jobStatus;
    private ArkApi users;
    private ArkApi tables;
    private ArkApi userTables;
    private ArkApi database;
    private ArkApi lastScn;

    private ArkApi tableHash;
    private ArkApi total;
    private ArkApi rowHash;
    private ArkApi analysis;
    private ArkApi getFile;
    private ArkApi recvFile;

    private ArkApi deleteConfig;
    private ArkApi getConfig;
    private ArkApi postConfig;
    private ArkApi getConfigList;

    private ArkApi logString;

    /**
     * Connect to the server.
     * 
     * @param address http uri (e.g. http://127.0.0.1:8080)
     */
    public ArkHttpApi(String address) {
        init(address);
    }

    /**
     * Connect to the server and authenticate.
     * 
     * @param address HTTP URI (e.g. http://127.0.0.1:8080)
     * @param password plain text password
     * @throws IOException if an I/O error occurs
     * @throws NoSuchAlgorithmException if no Provider supports a MessageDigestSpi implementation for the
     *             specified algorithm.
     */
    public ArkHttpApi(String address, String password) throws IOException, NoSuchAlgorithmException {

        init(address);
        open(password);
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.io.Closeable#close()
     */
    @Override
    public void close() throws IOException {
        httpClient.close();
    }

    /**
     * POST /afc/auth/init
     * 
     * @param password plain text password
     * @return true if authentication successful, otherwise false
     * @throws IOException if an I/O error occurs
     * @throws NoSuchAlgorithmException if no Provider supports a MessageDigestSpi implementation for the
     *             specified algorithm.
     */
    public boolean open(String password) throws IOException, NoSuchAlgorithmException {

        password = password.charAt(0) + password + password.charAt(password.length() - 1);
        byte[] bytes = Base64.getEncoder().encode(getSHA256(password).getBytes());
        String requestJson = "{\"id\":\"admin\",\"password\":\"" + new String(bytes) + "\"}";
        String execute = execute(authInit.getRequest(requestJson));

        execute.isEmpty();

        String[] split = execute.split("\"");
        if (split[1].equals("token")) {
            authorization = split[3];
            return true;
        } else {
            authorization = null;
            return false;
        }

    }

    /**
     * POST /afc/module/start
     * 
     * @param json request entity
     * @return response
     * @throws IOException if an I/O error occurs
     */
    public String moduleStart(String json) throws IOException {

        return execute(moduleStart.getRequest(json));
    }

    /**
     * POST /afc/module/stop
     * 
     * @param json request entity
     * @return response
     * @throws IOException if an I/O error occurs
     */
    public String moduleStop(String json) throws IOException {

        return execute(moduleStop.getRequest(json));
    }

    /**
     * GET /afc/module/state
     * 
     * @param type request param
     * @param alias request param
     * @param option request param
     * @return response
     * @throws IOException if an I/O error occurs
     */
    public String getModuleState(String type, String alias, String option) throws IOException {

        return execute(moduleState.getRequest("?type=" + type + "&alias=" + alias + "&option=" + option));
    }

    /**
     * GET /afc/info/peer
     * 
     * @return response
     * @throws IOException if an I/O error occurs
     */
    public String getPeer() throws IOException {

        return execute(peer.getRequest());
    }

    /**
     * GET /afc/info/modules
     * 
     * @return response
     * @throws IOException if an I/O error occurs
     */
    public String getModules() throws IOException {

        return execute(modules.getRequest());
    }

    /**
     * GET /afc/info/df
     * 
     * @param type request param
     * @param alias request param
     * @return response
     * @throws IOException if an I/O error occurs
     */
    public String getDiskFree(String type, String alias) throws IOException {

        return execute(diskFree.getRequest("?type=" + type + "&alias=" + alias));
    }

    /**
     * GET /afc/info/statistics
     * 
     * @param alias request param
     * @return response
     * @throws IOException if an I/O error occurs
     */
    public String getStatistics(String alias) throws IOException {

        return execute(statistics.getRequest("?alias=" + alias));
    }

    /**
     * GET /afc/info/tracingfile/list
     * 
     * @param alias request param
     * @return response
     * @throws IOException if an I/O error occurs
     */
    public String getTracingFileList(String alias) throws IOException {

        return execute(tracingFileList.getRequest("?alias=" + alias));
    }

    /**
     * GET /afc/info/job/status
     * 
     * @param jobCode request param
     * @return response
     * @throws IOException if an I/O error occurs
     */
    public String getJobStatus(String jobCode) throws IOException {

        return execute(jobStatus.getRequest("?job_code=" + jobCode));
    }

    /**
     * GET /afc/info/users
     * 
     * @param option request param
     * @return response
     * @throws IOException if an I/O error occurs
     */
    public String getUsers(String option) throws IOException {

        return execute(users.getRequest("?option=" + option));
    }

    /**
     * GET /afc/info/tables
     * 
     * @param user request param
     * @return response
     * @throws IOException if an I/O error occurs
     */
    public String getTables(String user) throws IOException {

        return execute(tables.getRequest("?user=" + user));
    }

    /**
     * GET /afc/info/usertables
     * 
     * @param option request param
     * @return response
     * @throws IOException if an I/O error occurs
     */
    public String getUserTables(String option) throws IOException {

        return execute(userTables.getRequest("?option=" + option));
    }

    /**
     * GET /afc/info/db
     * 
     * @param moduleName request param
     * @return response
     * @throws IOException if an I/O error occurs
     */
    public String getDatabase(String moduleName) throws IOException {

        return execute(database.getRequest("?module_name=" + moduleName));
    }

    /**
     * GET /afc/info/lastscn
     * 
     * @param postModuleName request param
     * @return response
     * @throws IOException if an I/O error occurs
     */
    public String getLastScn(String postModuleName) throws IOException {

        return execute(lastScn.getRequest("?post_module_name=" + postModuleName));
    }

    /**
     * GET /afc/command/compare/tablehash
     * 
     * @param userName request param
     * @param tableName request param
     * @param scn request param
     * @return response
     * @throws IOException if an I/O error occurs
     */
    public String getTableHash(String userName, String tableName, String scn) throws IOException {

        return execute(tableHash
                .getRequest("?user_name=" + userName + "&table_name=" + tableName + "&scn=" + scn));
    }

    /**
     * POST /afc/command/compare/total
     * 
     * @param json request entity
     * @return response
     * @throws IOException if an I/O error occurs
     */
    public String totalCompare(String json) throws IOException {

        return execute(total.getRequest(json));
    }

    /**
     * POST /afc/command/compare/rowhash
     * 
     * @param json request entity
     * @return response
     * @throws IOException if an I/O error occurs
     */
    public String rowCompare(String json) throws IOException {

        return execute(rowHash.getRequest(json));
    }

    /**
     * POST /afc/command/compare/rowcompareanalysis
     * 
     * @param json request entity
     * @return response
     * @throws IOException if an I/O error occurs
     */
    public String analysisRow(String json) throws IOException {

        return execute(analysis.getRequest(json));
    }

    /**
     * GET /afc/command/getfile
     * 
     * @param sourcePath request param
     * @return response
     * @throws IOException if an I/O error occurs
     */
    public String getFile(String sourcePath) throws IOException {

        return execute(getFile.getRequest("?source_path=" + sourcePath));
    }

    /**
     * POST /afc/command/recvfile
     * 
     * @param json request entity
     * @return response
     * @throws IOException if an I/O error occurs
     */
    public String recvFile(String json) throws IOException {

        return execute(recvFile.getRequest(json));
    }

    /**
     * DELETE /afc/config
     * 
     * @param type request param
     * @param alias request param
     * @return response
     * @throws IOException if an I/O error occurs
     */
    public String deleteConfig(String type, String alias) throws IOException {

        return execute(deleteConfig.getRequest("?type=" + type + "&alias=" + alias));
    }

    /**
     * GET /afc/config
     * 
     * @param type request param
     * @param alias request param
     * @return response
     * @throws IOException if an I/O error occurs
     */
    public String getConfig(String type, String alias) throws IOException {

        return execute(getConfig.getRequest("?type=" + type + "&alias=" + alias));
    }

    /**
     * POST /afc/config
     * 
     * @param json request entity
     * @return response
     * @throws IOException if an I/O error occurs
     */
    public String setConfig(String json) throws IOException {

        return execute(postConfig.getRequest(json));
    }

    /**
     * GET /afc/config/list
     * 
     * @param type request param
     * @return response
     * @throws IOException if an I/O error occurs
     */
    public String getConfigList(String type) throws IOException {

        return execute(getConfigList.getRequest("?type=" + type));
    }

    /**
     * GET /afc/log/logstring
     * 
     * @param fileName request param
     * @param start request param
     * @return response
     * @throws IOException if an I/O error occurs
     */
    public String getLogs(String fileName, String start) throws IOException {

        return execute(logString.getRequest("?filename=" + fileName + "&start=" + start));
    }

    private void init(String address) {

        httpClient = HttpClients.custom().addInterceptorLast(new HttpRequestInterceptor() {

            public void process(final HttpRequest request, final HttpContext context)
                    throws HttpException, IOException {

                request.addHeader("Authorization", authorization);
            }

        }).build();

        authInit = new ArkApi(address + "/afc/auth/init", Method.POST);
        moduleStart = new ArkApi(address + "/afc/module/start", Method.POST);
        moduleStop = new ArkApi(address + "/afc/module/stop", Method.POST);
        moduleState = new ArkApi(address + "/afc/module/state", Method.GET);

        peer = new ArkApi(address + "/afc/info/peer", Method.GET);
        modules = new ArkApi(address + "/afc/info/modules", Method.GET);
        diskFree = new ArkApi(address + "/afc/info/df", Method.GET);
        statistics = new ArkApi(address + "/afc/info/statistics", Method.GET);
        tracingFileList = new ArkApi(address + "/afc/info/tracingfile/list", Method.GET);
        jobStatus = new ArkApi(address + "/afc/info/job/status", Method.GET);
        users = new ArkApi(address + "/afc/info/users", Method.GET);
        tables = new ArkApi(address + "/afc/info/tables", Method.GET);
        userTables = new ArkApi(address + "/afc/info/usertables", Method.GET);
        database = new ArkApi(address + "/afc/info/db", Method.GET);
        lastScn = new ArkApi(address + "/afc/info/lastscn", Method.GET);

        tableHash = new ArkApi(address + "/afc/command/compare/tablehash", Method.GET);
        total = new ArkApi(address + "/afc/command/compare/total", Method.POST);
        rowHash = new ArkApi(address + "/afc/command/compare/rowhash", Method.POST);
        analysis = new ArkApi(address + "/afc/command/compare/rowcompareanalysis", Method.POST);
        getFile = new ArkApi(address + "/afc/command/compare/getfile", Method.GET);
        recvFile = new ArkApi(address + "/afc/command/compare/recvfile", Method.POST);

        deleteConfig = new ArkApi(address + "/afc/config", Method.DELETE);
        getConfig = new ArkApi(address + "/afc/config", Method.GET);
        postConfig = new ArkApi(address + "/afc/config", Method.POST);
        getConfigList = new ArkApi(address + "/afc/config/List", Method.GET);

        logString = new ArkApi(address + "/afc/log/logstring", Method.GET);

    }

    private String execute(HttpUriRequest request) throws IOException {

        try (CloseableHttpResponse response = httpClient.execute(request)) {

            HttpEntity entity = response.getEntity();
            return EntityUtils.toString(entity);
            // EntityUtils.consume(entity);
        }
    }

    private String getSHA256(String planText) throws NoSuchAlgorithmException {

        StringBuffer sb = new StringBuffer();

        MessageDigest messageDigest = MessageDigest.getInstance("SHA-256");
        messageDigest.update(planText.getBytes());
        byte bytes[] = messageDigest.digest();
        for (byte byte_ : bytes) {
            sb.append(Integer.toString((byte_ & 0xff) + 0x100, 16).substring(1));
        }

        return sb.toString();
    }

}
