package com.dbs.drnd.ark.cdc.http;

import java.io.Closeable;
import java.io.IOException;

import org.apache.http.HttpEntity;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpDelete;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.methods.HttpPut;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;

/**
 * HTTP Client.
 * 
 * @author jinwoo
 *
 */
public class ArkHttpClient implements Closeable {

    private final String address;
    private final CloseableHttpClient httpClient;

    /**
     * Connect to the server.
     * 
     * @param address HTTP URI (e.g. http://127.0.0.1:8080)
     */
    public ArkHttpClient(String address) {

        this.address = address;
        this.httpClient = HttpClients.createDefault();
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.io.Closeable#close()
     */
    @Override
    public void close() throws IOException {
        httpClient.close();
    }

    /**
     * Execute the GET request.
     * 
     * @param uri HTTP URI
     * @return response
     * @throws IOException if an I/O error occurs
     */
    public String get(String uri) throws IOException {

        String result = null;
        HttpGet httpGet = new HttpGet(this.address + uri);

        try (CloseableHttpResponse response = httpClient.execute(httpGet)) {

            HttpEntity entity = response.getEntity();
            result = EntityUtils.toString(entity);
            // EntityUtils.consume(entity);

        }

        return result;
    }

    /**
     * Execute the POST request.
     * 
     * @param url HTTP URI
     * @param json request entity
     * @return response
     * @throws IOException if an I/O error occurs
     */
    public String post(String url, String json) throws IOException {

        String result = null;
        HttpPost httpPost = new HttpPost(this.address + url);
        httpPost.setEntity(new StringEntity(json, ContentType.create("application/json", "UTF-8")));

        try (CloseableHttpResponse response = httpClient.execute(httpPost)) {

            HttpEntity entity = response.getEntity();
            result = EntityUtils.toString(entity);
            // EntityUtils.consume(entity);

        }

        return result;
    }

    /**
     * Execute the PUT request.
     * 
     * @param url HTTP URI
     * @param json request entity
     * @return response
     * @throws IOException if an I/O error occurs
     */
    public String put(String url, String json) throws IOException {

        String result = null;
        HttpPut httpPut = new HttpPut(this.address + url);
        httpPut.setEntity(new StringEntity(json, ContentType.create("application/json", "UTF-8")));

        try (CloseableHttpResponse response = httpClient.execute(httpPut)) {

            HttpEntity entity = response.getEntity();
            result = EntityUtils.toString(entity);
            // EntityUtils.consume(entity);

        }

        return result;
    }

    /**
     * Execute the DELETE request.
     * 
     * @param url HTTP URI
     * @return response
     * @throws IOException if an I/O error occurs
     */
    public String delete(String url) throws IOException {

        String result = null;
        HttpDelete httpDelete = new HttpDelete(this.address + url);

        try (CloseableHttpResponse response = httpClient.execute(httpDelete)) {

            HttpEntity entity = response.getEntity();
            result = EntityUtils.toString(entity);
            // EntityUtils.consume(entity);

        }

        return result;
    }

}
